%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
% DYNARE CODE FOR THE MODEL UNDER THE REFERENCE REGIME, i.e. BINDING CONSTRAINT
%  William Gatt
% 
%  -- used to run model over different parameters

@#define iterate_params = 1   

% Declare variables and shocks
     @#include "variables_and_shocks.mod"

% Calibrate model 
     @#include "param_calib.mod"
     
     @#if iterate_params 
        @#include "param_calib1.mod"
     @#endif
 
% Declare the model equations
    @#include "model_equations.mod"

% Add relevant equations to model 
    model;
       [name = 'Notional debt']
       B_NOT = (1-gamma)*(m*N_H_W) + gamma*B_NOT(-1)/PI;
       [name = 'Net housing wealth -- borrowers']
       N_H_W = ((q_1*PI_1)/R)*Hb; 
       [name = 'Actual debt']
       B = B_NOT;
       [name = 'Leverage limit']
       MaxLev = B - B_NOT;

       [name = 'Macroprudential policy rule']
       m = (m_ss^(1-rho_m))*(((B_4Y)/STEADY_STATE(B_4Y))^((-delta_m)*(1-rho_m)))*(m(-1)^(rho_m));
%        m = (m_ss^(1-rho_m))*(((B_4Y)/STEADY_STATE(B_4Y))^((-delta_m)*(1-rho_m)))*(m(-1)^(rho_m))*m_shock;    
     end;

% Initialise SS solver
    @#include "model_initval_ss.mod"

% Calibrate shocks
    @#include "shock_calibration.mod"

    options_.noprint=1;
    steady(solve_algo=3); 

check; 
% model_diagnostics;

stoch_simul(order=1,nocorr,nomoments,irf=0,noprint,nograph);
