%% set inputs for solution below
%  The program produces responses to the shocks selected below under
%  irfshock. Paths for all the endogenous variables in the model selected
%  are produced. VariableName_difference holds the piece-wise linear solution for
%  VariableName.  VariableName_uncdifference holds the linear solution for
%  VariableName.

% clear all ; 
% clc;
% close all



global M_ oo_ 

    % Model regimes
%     regime_11       = 'Mod_borrconst_binding_sym_pf'; % constraint BINDS, Credit BOOM
%     regime_01       = 'Mod_borrconst_slack_sym_pf';   % constraint SLACK, Credit BOOM
        
    regime_11       = 'Mod_borrconst_binding_sym'; % constraint BINDS, Credit BOOM
    regime_01       = 'Mod_borrconst_slack_sym';   % constraint SLACK, Credit BOOM
        
    constraint1          =   'mju<-mju_ss';     % escape from reference model - constraint becomes slack
    constraint_relax1    =   'MaxLev > 0';      % back to reference model     - constraint becomes binding
 
    irfshock =char('eps_j');      % label for innovation for IRFs

    maxiter = 20;
    nperiods = 200;
    
%     j_ss = 0.06;
%     j_vec   = linspace(-4*j_ss,4*j_ss,13);    % Make sure a value in j_vec is centered around 0 to make it easier to plot and make sense of the kink

%     tic;
    
   
fprintf('\n\n=================================================== \nSolving model using OCCBIN \n - Model with OBC in borrowing \n===================================================\n');
% pause(2);

% Pre-allocation
%         grdsize  =  length(j_vec);
        grdsize  =  length(eps_vec);
        
        Cb_PF    =   zeros(grdsize,1);
        Cs_PF    =   zeros(grdsize,1);
        q_PF     =   zeros(grdsize,1);
        Y_PF     =   zeros(grdsize,1);
        B_4Y_PF  =   zeros(grdsize,1);
        B_PF     =   zeros(grdsize,1);
        Hs_PF    =   zeros(grdsize,1);
        Hb_PF    =   zeros(grdsize,1);
        PI_PF    =   zeros(grdsize,1);
        r4_PF    =   zeros(grdsize,1);
        m_PF     =   zeros(grdsize,1);
        mju_state=   zeros(grdsize,1);
        j_state  =   zeros(grdsize,1);
        
       
        % IRFs
        Cb_irf    =   zeros(nperiods,grdsize);
        Cs_irf    =   zeros(nperiods,grdsize);
        q_irf     =   zeros(nperiods,grdsize);
        Y_irf     =   zeros(nperiods,grdsize);
        B_4Y_irf  =   zeros(nperiods,grdsize);
        B_irf     =   zeros(nperiods,grdsize);
        Hs_irf    =   zeros(nperiods,grdsize);
        Hb_irf    =   zeros(nperiods,grdsize);
        PI_irf    =   zeros(nperiods,grdsize);
        r4_irf    =   zeros(nperiods,grdsize);
        m_irf     =   zeros(nperiods,grdsize);
        mju_irf   =   zeros(nperiods,grdsize);
        

% for index_j=1:numel(j_vec)    % loop over shock value
for index_j=1:numel(eps_vec)    % loop over shock value

    %     for index_Cb=1:numel(Cb_vec)  % loop over variable value

%         shockssequence = [ j_vec(index_j) ]
        shockssequence = [ eps_vec(index_j) ]
        
        [zdatalinear, zdatapiecewise, zdatass, oobase_, Mbase_  ] = solve_one_constraint(...
        regime_11,regime_01,...
        constraint1,constraint_relax1,...
        shockssequence,irfshock,nperiods);
 

% [zdatalinear zdatapiecewise zdatass oobase_ Mbase_] = solve_two_constraints(...
%   regime_11,regime_01,regime_10,regime_00,...
%   constraint1, constraint2,...
%   constraint_relax1, constraint_relax2,...
%   shockssequence,irfshock,nperiods);
  
%         fprintf('\n\nSimulation complete! ... extracting IRFs...\n');

        % unpack the IRFs
        for i=1:Mbase_.endo_nbr
          eval([deblank(Mbase_.endo_names{i,:}),'_l=zdatalinear(:,i);']);
          eval([deblank(Mbase_.endo_names{i,:}),'_p=zdatapiecewise(:,i);']);
          eval([deblank(Mbase_.endo_names{i,:}),'_ss=zdatass(i);']);
        end
     
        % Policy functions
        Cb_PF(index_j,:)    =   Cb_p(1)+Cb_ss;
        Cs_PF(index_j,:)    =   Cs_p(1)+Cs_ss;
        q_PF(index_j,:)     =   q_p(1)+q_ss;
        Y_PF(index_j,:)     =   Y_p(1)+Y_ss;
        B_4Y_PF(index_j,:)  =   B_4Y_p(1)+B_4Y_ss;
        B_PF(index_j,:)     =   B_p(1)+B_ss;
        Hs_PF(index_j,:)    =   Hs_p(1)+Hs_ss;
        Hb_PF(index_j,:)    =   Hb_p(1)+Hb_ss;
        PI_PF(index_j,:)    =   PI_p(1)+PI_ss;
        r4_PF(index_j,:)    =   r4_p(1)+r4_ss;
        m_PF(index_j,:)     =   m_p(1)+m_ss;        

        j_state(index_j,:) = j_p(1)+j_ss;
        mju_state(index_j,:)=   mju_p(1)+mju_ss;

%         % Bad (naive) policy functions based on linear approx.
%         Cb_PF_bad(index_j,:)    =   Cb_l(1)+Cb_ss;
%         Cs_PF_bad(index_j,:)    =   Cs_l(1)+Cs_ss;
%         q_PF_bad(index_j,:)     =   q_l(1)+q_ss;
%         Y_PF_bad(index_j,:)     =   Y_l(1)+Y_ss;
%         B_4Y_PF_bad(index_j,:)  =   B_4Y_l(1)+B_4Y_ss;
%         B_PF_bad(index_j,:)     =   B_l(1)+B_ss;
%         Hs_PF_bad(index_j,:)    =   Hs_l(1)+Hs_ss;
%         Hb_PF_bad(index_j,:)    =   Hb_l(1)+Hb_ss;
%         PI_PF_bad(index_j,:)    =   PI_l(1)+PI_ss;
%         r4_PF_bad(index_j,:)    =   r4_l(1)+r4_ss;
%         m_PF_bad(index_j,:)     =   m_l(1)+m_ss;

%         j_state_bad(index_j,:)  = j_l(1)+j_ss;
%         mju_state_bad(index_j,:)= mju_l(1)+mju_ss;

        
        % IRFs
        Cb_irf(:,index_j)   =   (Cb_p/Cb_ss)*100;
        Cs_irf(:,index_j)   =   (Cs_p/Cs_ss)*100;
        Hb_irf(:,index_j)   =   (Hb_p/Hb_ss)*100;
        Hs_irf(:,index_j)   =   (Hs_p/Hs_ss)*100;
        q_irf(:,index_j)    =   (q_p/q_ss)*100;
        Y_irf(:,index_j)    =   (Y_p/Y_ss)*100;
        r4_irf(:,index_j)   =   r4_p+r4_ss;
        m_irf(:,index_j)    =   m_p+m_ss;
        mju_irf(:,index_j)  =   mju_p+mju_ss;
        PI_irf(:,index_j)   =   (PI_p/PI_ss)*100;
        B_4Y_irf(:,index_j) =   (B_4Y_p/B_4Y_ss)*100;
        B_irf(:,index_j)    =   (B_p/B_ss)*100;

%         Cb_irf_bad(:,index_j)   =   (Cb_l/Cb_ss)*100;
%         Cs_irf_bad(:,index_j)   =   (Cs_l/Cs_ss)*100;
%         Hb_irf_bad(:,index_j)   =   (Hb_l/Hb_ss)*100;
%         Hs_irf_bad(:,index_j)   =   (Hs_l/Hs_ss)*100;
%         q_irf_bad(:,index_j)    =   (q_l/q_ss)*100;
%         Y_irf_bad(:,index_j)    =   (Y_l/Y_ss)*100;
%         r4_irf_bad(:,index_j)   =   r4_l+r4_ss;
%         m_irf_bad(:,index_j)    =   m_l+m_ss;
%         mju_irf_bad(:,index_j)  =   mju_l+mju_ss;
%         PI_irf_bad(:,index_j)   =   (PI_l/PI_ss)*100;
%         B_4Y_irf_bad(:,index_j) =   (B_4Y_l/B_4Y_ss)*100;
%         B_irf_bad(:,index_j)    =   (B_l/B_ss)*100;
        
 end
% end
    
%     fprintf(' \nBorrowing constraint is slack in %2.0f quarters\n', sum(mju_p/(-mju_ss)>0.999));  %when constraint is slack, mju_p is *almost* equal to -mju_ss

%     etime=toc;
%     fprintf('...total simulation time: %4.2f seconds\n', etime);

%% POLICY FUNCTIONS
return
    figure('Name','Policy functions');
    statevar = (Hb_PF/Hb_ss-1)*100;

    subplot(2,2,1); grid on; hold on;
    plot(statevar,(mju_state/mju_ss-1)*100,'lineStyle','-','marker','none','linewidth',2); hold on  
    xlabel('Housing (borrowers), % deviation from ss')  
    title('Lagrange multiplier, % from ss');

    subplot(2,2,2); grid on; hold on;
    plot(statevar,(Y_PF/Y_ss-1)*100,'lineStyle','-','marker','none','linewidth',2); hold on  
    xlabel('Housing (borrowers), % deviation from ss')  
    title('Output, % from ss');

    subplot(2,2,3); grid on; hold on;
    plot(statevar,(Cb_PF/Cb_ss-1)*100,'lineStyle','-','marker','none','linewidth',2); hold on
    xlabel('Housing (borrowers), % deviation from ss')  
    title('Consumption - borrowers, % from ss');

    subplot(2,2,4); grid on; hold on;
    plot(statevar,(Cs_PF/Cs_ss-1)*100,'lineStyle','-','marker','none','linewidth',2); hold on
    xlabel('Housing (borrowers), % deviation from ss')  
    title('Consumption - savers, % from ss');

%%
return
irfperiods=40;
fntsize = 14;

set(0,'DefaultLineLineWidth',2);

figure;
subplot(2,4,1); grid on; hold on;
h3=plot(q_p(1:irfperiods)/q_ss*100);
h4=plot(q_l(1:irfperiods)/q_ss*100); %axis tight;
title('$q$','interpreter','latex','Fontsize',20);

subplot(2,4,2); grid on; hold on;
h1=plot(Cb_p(1:irfperiods)/Cb_ss*100);
h2=plot(Cb_l(1:irfperiods)/Cb_ss*100);
title('$C_{b}$','interpreter','latex','Fontsize',fntsize);

subplot(2,4,3); grid on; hold on;
h9=plot(Y_p(1:irfperiods)/Y_ss*100);
h10=plot(Y_l(1:irfperiods)/Y_ss*100);
title('$Y$','interpreter','latex','Fontsize',fntsize);

subplot(2,4,4); grid on; hold on;
h5=plot(B_4Y_p(1:irfperiods)+B_4Y_ss);
h6=plot(B_4Y_l(1:irfperiods)+B_4Y_ss); %axis tight;
title('$B4Y$','interpreter','latex','Fontsize',fntsize);

subplot(2,4,5); grid on; hold on;
h7=plot(mju_p(1:irfperiods)+mju_ss);
h8=plot(mju_l(1:irfperiods)+mju_ss); %axis tight;
title('$\mu$','interpreter','latex','Fontsize',fntsize);
legend({'Piecewise linear','Linear'},'Fontsize',10,'Location','SouthEast');

subplot(2,4,6); grid on; hold on;
h9=plot(r4_p(1:irfperiods));
h10=plot(r4_l(1:irfperiods));
title('$R$ (ann.)','interpreter','latex','Fontsize',fntsize);

subplot(2,4,7); grid on; hold on;
h9=plot(Hb_p(1:irfperiods)+m_ss);
h10=plot(Hb_l(1:irfperiods)+m_ss);
title('$H_{b}$','interpreter','latex','Fontsize',fntsize);

subplot(2,4,8); grid on; hold on;
h9=plot(PI_p(1:irfperiods)/PI_ss*100);
h10=plot(PI_l(1:irfperiods)/PI_ss*100);
title('$\pi$','interpreter','latex','Fontsize',fntsize);
